/*
TB6600, Titre : Contrôle de la vitesse et de la direction d'un moteur pas-à-pas avec Arduino
Code créé le 2023-10-03 avec l'IDE Arduino version 2.0.3
Configuration du TB6600 pour les micropas : SW1=ON  SW2=OFF  SW3=OFF
Réalisé par Hervé Mazelin pour RedOhm

31/08/2021
adapatation à module "Arduino pro mini" by paulfjujo
COM 25
modif EnPin  ON => motor ON
*/

// Définition des broches utilisées pour le contrôle du driver
const int PULS_PIN = 10;  // Broche pour générer les impulsions   14 blanc
const int DIR_PIN = 11;  // Broche pour définir la direction      15 vert
const int ENA_PIN = 12;   // Broche pour ON/OF moteur             16 noir

// Définition de la tempo en microsecondes
int TEMPO = 250;
int nouvelle_tempo = 0;
// Variable pour stocker la valeur de tempo extraite de la chaîne saisie par l'utilisateur
String valeur_alpha_nouvelle_tempo;
String chaine_caractere;
int i=0;
int j=0;
// Variable pour stocker le temps précédent
unsigned long tempsPrecedent = 0;


// Cette fonction est exécutée une seule fois au démarrage de l'Arduino.
void setup() {
  // Configuration des pattes en sortie
  pinMode(ENA_PIN, OUTPUT);
  pinMode(PULS_PIN, OUTPUT);
  pinMode(DIR_PIN, OUTPUT);
  digitalWrite(PULS_PIN, LOW);


  // Initialisation de la communication série
  Serial.begin(19200);
  Serial.println("");
  Serial.println(" Version 01/09/2024 Arduino Pro mini, by RedOhm , rev paulfjujo\r\n");
 

/*
// optional pour tester les sortie ...
 Serial.println(" Check optionneletats des pins 10,11,12 avec oscillo\r\n");

do
{
 Serial.print("j=");
 Serial.println(j);
 Serial.println(" Test PULS_PIN 9\r\n");
for (i=0;i<100;i++)
 {
   digitalWrite(PULS_PIN, HIGH);
   delay(5);
    digitalWrite(PULS_PIN, LOW);
   delay(5);
  }

 Serial.println(" Test DIR_PIN 10\r\n");
for (i=0;i<100;i++)
 {
   digitalWrite(DIR_PIN, HIGH);
   delay(10);
    digitalWrite(DIR_PIN, LOW);
   delay(10);
  }

   Serial.println(" Test ENA_PIN 11\r\n");
for (i=0;i<100;i++)
 {
   digitalWrite(ENA_PIN, HIGH);
   delay(15);
    digitalWrite(ENA_PIN, LOW);
   delay(15);
  }
  delay(1500);
  Serial.println(" -----\r\n");
  j++;
} while(j<10);
*/

  delay(10);

  // Direction LOW antihoraire / HIGH horaire
  digitalWrite(DIR_PIN, HIGH);
  delayMicroseconds(15);

  // Valide le driver : Active LOW / Desactive HIGH
  //digitalWrite(ENA_PIN, LOW);
  // NON, c'est le contraire !
   digitalWrite(ENA_PIN, HIGH);
   Serial.println(" set ENPin High  ");
   delayMicroseconds(15);

  // Affichage du message pour saisir une nouvelle valeur de tempo
  Serial.println("Saisir une nouvelle commande  ");
  Serial.println("");
 
  
}

// Cette fonction est exécutée en boucle sans fin après le démarrage de l'Arduino.
void loop() {


  // Si des données sont disponibles sur la communication série
  if (Serial.available() > 0) {
    // lit une chaîne de caractères entrée sur le port série de l'Arduino et stockée dans la
    // variable chaine_caractere.
    chaine_caractere = Serial.readStringUntil('\n');
    // Affichage de la chaîne de caractères entrée par l'utilisateur sur le port série de l'Arduino.
    Serial.println("La chaine saisie est : " + chaine_caractere);

    // Recherche la lettre 'A' dans la chaîne de caractères qui defini le sens Horaire
    int indexA = chaine_caractere.indexOf('A');
    int indexa = chaine_caractere.indexOf('a');
    // Recherche la lettre 'R' dans la chaîne de caractères qui defini le sens anti Horaire
    int indexR = chaine_caractere.indexOf('R');
    int indexr = chaine_caractere.indexOf('r');
    // Si la lettre 'A' ou 'a' est trouvée dans la chaîne de caractères
    if (indexA != -1 || indexa != -1) {
      // Direction horaire
      digitalWrite(DIR_PIN, HIGH);
      delayMicroseconds(5);
      //Ce code permet de regrouper les indices de la lettre "A" en majuscule et de
      //la lettre "a" en minuscule dans la chaîne de caractères "chaine_caractere".
      //La variable "indexA" stocke l'index de la première occurrence de la lettre "A" en
      //majuscule et la variable "indexa" stocke l'index de la première occurrence de la lettre
      //"a" en minuscule.
      //La variable "index_AV_regroupe" est ensuite initialisée avec la somme
      //de ces deux variables, permettant de regrouper les deux indices dans une seule variable.
      //Cette opération est utile pour faciliter l'extraction de la valeur numérique de vitesse
      //à partir de la chaîne de caractères saisie.
      int index_AV_regroupe = indexA + indexa;
      valeur_alpha_nouvelle_tempo = (chaine_caractere.substring(index_AV_regroupe + 2));
      Serial.print("Valeur de indexA " + String(indexA));
      Serial.print(" Valeur de indexa " + String(indexa));
      Serial.println(" = Valeur de l'index de regroupement " + String(index_AV_regroupe));
      Serial.println("Demande de marche avant & valeur de vitesse =>  " + valeur_alpha_nouvelle_tempo);
      Serial.println("Saisir une nouvelle commande  ");
      Serial.println("");
      //Ce code permet de convertir la valeur de vitesse stockée sous forme de chaîne de caractères
      //dans la variable "valeur_alpha_nouvelle_tempo" en un entier. La fonction "toInt()"
      //permet de réaliser cette conversion. La variable "TEMPO" est alors mise à jour avec
      //cette nouvelle valeur de vitesse.
      TEMPO = valeur_alpha_nouvelle_tempo.toInt();
    } else if (indexR != -1 || indexr != -1) {
      // Direction LOW antihoraire
      digitalWrite(DIR_PIN, LOW);
      delayMicroseconds(5);
      int index_AR_regroupe = indexR + indexr;
      valeur_alpha_nouvelle_tempo = (chaine_caractere.substring(index_AR_regroupe + 2));
      Serial.println("Demande de marche arriere & valeur de vitesse => " + valeur_alpha_nouvelle_tempo);
      Serial.println("Saisir une nouvelle commande  ");
      Serial.println("");
      TEMPO = valeur_alpha_nouvelle_tempo.toInt();
    }

    else {
      Serial.println("Il manque la confirmation de sens de direction La lettre 'a','A','r','R' n'a pas été trouvée.");
      Serial.println("Saisir une nouvelle commande  ");
      Serial.println("");
    }
  }


  // Récupération du temps actuel en microsecondes
  unsigned long tempsActuel = micros();
  // Si le temps écoulé depuis la dernière impulsion est supérieur ou égal à la valeur de la tempo
  if (tempsActuel - tempsPrecedent >= TEMPO) {
    tempsPrecedent = tempsActuel;

    // Inversion de l'état de la broche puls pour générer une impulsion
    digitalWrite(PULS_PIN, !digitalRead(PULS_PIN));
  }
}
(15:37:37.270)  Version 01/09/2024 Arduino Pro mini, by RedOhm , rev paulfjujo
(15:37:37.317)  Test Ã ©  OK avec valeur de 110  Ã    10000
(15:37:37.333) 
(15:37:37.333)  set ENPin High  
(15:37:37.349) Saisir une nouvelle commande  
(15:37:37.364) 
(15:37:42.442) A100
(15:37:42.567) La chaine saisie est : A100
(15:37:42.567) Demande de marche avant & valeur de vitesse =>  100

(15:37:42.567) Saisir une nouvelle commande : 
(15:37:51.929) A110
(15:37:52.070) La chaine saisie est : A110
(15:37:52.070) Demande de marche avant & valeur de vitesse =>  110

(15:37:52.070) Saisir une nouvelle commande : 
(15:37:59.745) A5000
(15:37:59.886) La chaine saisie est : A5000
(15:37:59.886) Demande de marche avant & valeur de vitesse =>  5000

(15:37:59.886) Saisir une nouvelle commande : 
(15:38:08.181) A120
(15:38:08.306) La chaine saisie est : A120
(15:38:08.306) Demande de marche avant & valeur de vitesse =>  120

(15:38:08.306) Saisir une nouvelle commande : 
(15:38:22.135) A10000
(15:38:22.260) La chaine saisie est : A10000
(15:38:22.260) Demande de marche avant & valeur de vitesse =>  10000

(15:38:22.260) Saisir une nouvelle commande : 
(15:41:40.385) Saisir une nouvelle commande : 
(15:41:42.244) R10000
(15:41:42.354) La chaine saisie est : R10000
(15:41:42.354) Demande de marche arriere & valeur de vitesse => 10000

(15:41:42.354) Saisir une nouvelle commande : 
(15:41:47.607) R1000
(15:41:47.763) La chaine saisie est : R1000
(15:41:47.763) Demande de marche arriere & valeur de vitesse => 1000

(15:41:47.763) Saisir une nouvelle commande : 


*/
